import { Component, inject, Inject, OnInit, PLATFORM_ID } from '@angular/core';
import { ApiAuthenticationService } from '../../shared/services/api/api-authentication.service';
import { MatCardModule } from '@angular/material/card';
import { MatSelectModule } from '@angular/material/select';
import { HttpClient } from '@angular/common/http';
import { isPlatformBrowser } from '@angular/common';
import { environment } from '../../../environments/environment';
import EventService from '../../shared/services/EventService';
import { HermesClientService } from '../../hermes-client.service';
import { Router } from '@angular/router';
import { timeout, first } from 'rxjs';
import ApiKey from '../../shared/models/api-key';
import { ApiKeyService } from '../../shared/services/api/api-key.service';

@Component({
  selector: 'impersonation',
  standalone: true,
  imports: [MatCardModule, MatSelectModule],
  templateUrl: './impersonation.component.html',
  styleUrl: './impersonation.component.scss'
})
export class ImpersonationComponent implements OnInit {
  private readonly keyService = inject(ApiKeyService);

  impersonated: string | undefined;
  users: { id: string, name: string }[];

  constructor(private hermes: HermesClientService, private auth: ApiAuthenticationService, private router: Router, private events: EventService, private http: HttpClient, @Inject(PLATFORM_ID) private platformId: Object) {
    this.users = []
  }

  ngOnInit(): void {
    if (!isPlatformBrowser(this.platformId) || !this.auth.isAdmin()) {
      return;
    }

    this.http.get(environment.API_HOST + '/admin/users', {
      headers: {
        'Authorization': 'Bearer ' + localStorage.getItem('jwt')
      }
    }).subscribe((data: any) => {
      this.users = data.filter((d: any) => d.name != this.auth.getUsername());
      const id = this.auth.getImpersonatedId();
      if (id && this.users.find(u => u.id == id)) {
        this.impersonated = id;
      }
    });

    if (this.auth.isAdmin()) {
      this.events.listen('impersonation', (userId) => {
        this.keyService.fetch(true)
          .pipe(timeout(3000), first())
          .subscribe(async (d: ApiKey[]) => {
            if (d.length > 0)
              this.hermes.login(d[0].id);
            await this.router.navigate([this.router.url.substring(1)]);
          });
      });
    }
  }

  public isAdmin() {
    return this.auth.isAdmin();
  }

  public getUsername() {
    return this.auth.getUsername();
  }

  public onChange(e: any) {
    if (!this.auth.isAdmin())
      return;

    if (!e.value) {
      this.http.delete(environment.API_HOST + '/admin/impersonate', {
        headers: {
          'Authorization': 'Bearer ' + localStorage.getItem('jwt')
        },
        body: {
          impersonation: e.value
        }
      }).subscribe(async (data: any) => {
        this.hermes.disconnect();
        this.events.emit('impersonation', e.value);
      });
    } else {
      this.http.put(environment.API_HOST + '/admin/impersonate', {
        impersonation: e.value
      }, {
        headers: {
          'Authorization': 'Bearer ' + localStorage.getItem('jwt')
        }
      }).subscribe(async (data: any) => {
        this.hermes.disconnect();
        this.events.emit('impersonation', e.value);
        await this.router.navigate(['tts-login']);
      });
    }
  }
}