import { Component, inject, Input } from '@angular/core';
import { Permission } from '../../shared/models/permission';
import { Group } from '../../shared/models/group';
import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { MatDialog } from '@angular/material/dialog';
import { HermesClientService } from '../../hermes-client.service';
import EventService from '../../shared/services/EventService';
import { PermissionItemEditComponent } from '../permission-item-edit/permission-item-edit.component';

@Component({
  selector: 'permission-item',
  imports: [
    MatButtonModule,
    MatIconModule,
  ],
  templateUrl: './permission-item.component.html',
  styleUrl: './permission-item.component.scss'
})
export class PermissionItemComponent {
  @Input({ required: true }) permission: Permission = { id: '', group_id: '', user_id: '', path: '', allow: null };
  @Input({ required: true }) group: Group | undefined;

  readonly dialog = inject(MatDialog);
  readonly client = inject(HermesClientService);
  readonly events = inject(EventService);

  responseError: string | undefined;
  waitForResponse = false;
  opened = false;


  delete() {
    if (this.opened || this.waitForResponse) {
      return;
    }

    this.waitForResponse = true;
    this.responseError = undefined;

    const permissionId = this.permission.id;

    this.client.first((d: any) => d.op == 4 && d.d.request.type == 'delete_group_permission' && d.d.request.data.id == permissionId)
      .subscribe({
        next: (d) => {
          if (d.d.error) {
            this.responseError = d.d.error;
          } else {
            this.events.emit('delete_group_permission', permissionId);
          }
        },
        error: () => this.responseError = 'Something went wrong.',
        complete: () => this.waitForResponse = false,
      });
    this.client.deleteGroupPermission(this.permission.id);
  }

  edit() {
    if (this.opened || this.waitForResponse) {
      return;
    }

    this.opened = true;

    const dialogRef = this.dialog.open(PermissionItemEditComponent, {
      data: { permission: this.permission, group: this.group, groups: [], isNew: false },
    });

    dialogRef.afterClosed().subscribe((permission: Permission) => {
      this.opened = false;
      if (!permission)
        return;

      this.permission.group_id = permission.group_id;
      this.permission.path = permission.path;
      this.permission.allow = permission.allow;
    });
  }
}
