import { inject, Injectable } from '@angular/core';
import { Observable, of, map } from 'rxjs';
import { HermesClientService } from '../../hermes-client.service';
import EventService from './EventService';
import { Group } from '../models/group';
import { GroupChatter } from '../models/group-chatter';

@Injectable({
  providedIn: 'root'
})
export default class GroupService {
  private readonly client = inject(HermesClientService);
  private readonly events = inject(EventService);
  private groups: Group[] = [];
  private chatters: GroupChatter[] = [];
  private loaded = false;
  createGroup$: Observable<any> | undefined;
  updateGroup$: Observable<any> | undefined;
  deleteGroup$: Observable<any> | undefined;

  createChatter$: Observable<any> | undefined;
  updateChatter$: Observable<any> | undefined;
  deleteChatter$: Observable<any> | undefined;

  constructor() {
    this.createGroup$ = this.client.filterByRequestType('create_group');
    this.updateGroup$ = this.client.filterByRequestType('update_group');
    this.deleteGroup$ = this.client.filterByRequestType('delete_group');

    this.createChatter$ = this.client.filterByRequestType('create_group_chatter');
    this.updateChatter$ = this.client.filterByRequestType('update_group_chatter');
    this.deleteChatter$ = this.client.filterByRequestType('delete_group_chatter');

    // Groups
    this.createGroup$?.subscribe(d => this.groups.push(d.data));
    this.updateGroup$?.subscribe(d => {
      const group = this.groups.find(g => g.id == d.data.id);
      if (group) {
        group.name = d.data.name;
        group.priority = d.data.priority;
      }
    });
    this.deleteGroup$?.subscribe(d => this.groups = this.groups.filter(r => r.id != d.request.data.id));

    // Chatters
    this.createChatter$?.subscribe(d => this.chatters.push(d.data));
    this.updateChatter$?.subscribe(d => {
      const chatter = this.chatters.find(g => g.group_id == d.data.group_id && g.chatter_id == d.data.chatter_id);
      if (chatter) {
        chatter.chatter_label = d.data.chatter_label;
        chatter.group_id = d.data.group_id;
      }
    });
    this.deleteChatter$?.subscribe(d => this.chatters = this.chatters.filter(r => r.group_id != d.request.data.group && r.chatter_id != d.request.data.chatter));

    this.events.listen('tts_logoff', () => {
      this.groups = [];
      this.chatters = [];
      this.loaded = false;
    });
  }


  fetch() {
    if (this.loaded) {
      return of({
        groups: this.groups,
        chatters: this.chatters,
      });
    }

    const $ = this.client.first(d => d.d.request.type == 'get_groups')!.pipe(map(d => d.d.data));
    $.subscribe(d => {
      this.groups = d.map((item: any) => item.group);
      this.chatters = d.map((item: any) => item.chatters).flat();
      this.loaded = true;
    });
    this.client.fetchGroups();
    return $.pipe(map((d: any) => ({
      groups: d.map((item: any) => item.group),
      chatters: d.map((item: any) => item.chatters).flat(),
    })));
  }
}
