import { inject, Injectable } from '@angular/core';
import { HermesClientService } from '../../hermes-client.service';
import EventService from './EventService';
import { Permission } from '../models/permission';
import { map, Observable, of } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class PermissionService {
  private readonly client = inject(HermesClientService);
  private readonly events = inject(EventService);
  private data: Permission[] = [];
  private loaded = false;
  create$: Observable<any> | undefined;
  update$: Observable<any> | undefined;
  delete$: Observable<any> | undefined;

  constructor() {
    this.create$ = this.client.filterByRequestType('create_group_permission');
    this.update$ = this.client.filterByRequestType('update_group_permission');
    this.delete$ = this.client.filterByRequestType('delete_group_permission');

    this.create$?.subscribe(d => {
      if (d.error) {
        return;
      }

      this.data.push(d.data);
    });
    this.update$?.subscribe(d => {
      if (d.error) {
        return;
      }

      const permission = this.data.find(p => p.id == d.data.id);
      if (permission) {
        permission.group_id = d.data.group_id;
        permission.path = d.data.path;
        permission.allow = d.data.allow;
        permission.user_id = d.data.user_id;
      }
    });
    this.delete$?.subscribe(d => {
      if (d.error) {
        return;
      }

      this.data = this.data.filter(r => r.id != d.request.data.id);
    });

    this.events.listen('tts_logoff', () => {
      this.data = [];
      this.loaded = false;
    });
  }


  fetch() {
    if (this.loaded) {
      return of(this.data);
    }

    const $ = this.client.first(d => d.d.request.type == 'get_group_permissions')!.pipe(map(d => d.d.data));
    $.subscribe(d => {
      this.data = d;
      this.loaded = true;
    });
    this.client.fetchPermissions();
    return $;
  }
}
