import { Component, inject, Inject, OnDestroy, OnInit, PLATFORM_ID } from '@angular/core';
import { FilterListComponent } from "../filter-list/filter-list.component";
import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { HermesClientService } from '../../hermes-client.service';
import { Filter, FilterFlag } from '../../shared/models/filter';
import { isPlatformBrowser } from '@angular/common';
import { ActivatedRoute, Router } from '@angular/router';
import { FilterItemEditComponent } from '../filter-item-edit/filter-item-edit.component';
import { MatDialog } from '@angular/material/dialog';
import TtsFilterService from '../../shared/services/tts-filter.service';
import { Subscription } from 'rxjs';

@Component({
  selector: 'filters',
  standalone: true,
  imports: [FilterListComponent, MatButtonModule, MatIconModule],
  templateUrl: './filters.component.html',
  styleUrl: './filters.component.scss'
})
export class FiltersComponent implements OnInit, OnDestroy {
  private readonly filterService = inject(TtsFilterService);
  private readonly route = inject(ActivatedRoute);
  private readonly subscriptions: Subscription[] = [];
  private isBrowser: boolean;
  readonly dialog = inject(MatDialog);
  items: Filter[];


  constructor(private client: HermesClientService, private router: Router, @Inject(PLATFORM_ID) private platformId: Object) {
    this.isBrowser = isPlatformBrowser(this.platformId);
    this.items = [];

    this.route.data.subscribe(data => {
      if (!data['filters'])
        return;

      this.items = [...data['filters']];
    });
    let subscription = this.filterService.create$?.subscribe(d => {
      if (d.error || d.request.nounce != null && d.request.nounce.startsWith(this.client.session_id))
        return;

      this.items.push(d.data);
    });
    if (subscription)
      this.subscriptions.push(subscription);

    subscription = this.filterService.update$?.subscribe(d => {
      if (d.error || d.request.nounce != null && d.request.nounce.startsWith(this.client.session_id))
        return;

      const filter = this.items.find(f => f.id == d.data.id);
      if (filter) {
        filter.search = d.data.search;
        filter.replace = d.data.replace;
        filter.flag = d.data.flag || 0;
      }
    });
    if (subscription)
      this.subscriptions.push(subscription);

    subscription = this.filterService.delete$?.subscribe(d => {
      if (d.error)
        return;

      this.items = this.items.filter(a => a.id != d.request.data.id);
    });
    if (subscription)
      this.subscriptions.push(subscription);
  }

  ngOnInit(): void {
    if (!this.isBrowser)
      return;

    if (!this.client.logged_in) {
      this.router.navigate(["tts-login"]);
      return;
    }
  }

  ngOnDestroy() {
    this.subscriptions.forEach(s => s.unsubscribe());
  }

  openDialog(): void {
    const dialogRef = this.dialog.open(FilterItemEditComponent, {
      data: { id: '', user_id: '', search: '', replace: '', flag: FilterFlag.None },
    });

    dialogRef.afterClosed().subscribe((result: Filter) => {
      if (result)
        this.items.push(result);
    });
  }
}