import { isPlatformBrowser } from '@angular/common';
import { HttpClient } from '@angular/common/http';
import { Component, Inject, OnInit, PLATFORM_ID } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { ApiAuthenticationService } from '../shared/services/api/api-authentication.service';
import { environment } from '../../environments/environment';

@Component({
  selector: 'app-twitch-auth-callback',
  standalone: true,
  imports: [],
  templateUrl: './twitch-auth-callback.component.html',
  styleUrl: './twitch-auth-callback.component.scss'
})
export class TwitchAuthCallbackComponent implements OnInit {
  private isBrowser: boolean;

  constructor(private http: HttpClient, private auth: ApiAuthenticationService, private route: ActivatedRoute, private router: Router, @Inject(PLATFORM_ID) private platformId: Object) {
    this.isBrowser = isPlatformBrowser(this.platformId)
  }

  async ngOnInit(): Promise<any> {
    if (!this.isBrowser) {
      return;
    }

    if (this.auth.isAuthenticated()) {
      await this.router.navigate(['tts-login']);
      return;
    }

    const code = this.route.snapshot.queryParamMap.get('code');
    const scope = this.route.snapshot.queryParamMap.get('scope');
    const state = this.route.snapshot.queryParamMap.get('state');

    if (!code || !scope || !state) {
      await this.router.navigate(['login'], {
        queryParams: {
          error: 'callback_issue'
        }
      });
      return;
    }

    this.http.post(environment.API_HOST + '/auth/twitch/callback', { code, scope, state })
      .subscribe(async (response: any) => {
        if (!response?.authenticated) {
          await this.router.navigate(['login'], {
            queryParams: {
              error: 'callback_issue'
            }
          });
          return;
        }

        localStorage.setItem('jwt', response.token);
        this.auth.update();

        await this.router.navigate(['tts-login']);
      });
  }
}
